﻿local debugMode = false

local guildRosterTable, rosterCooldown, delayedCall


local frame = CreateFrame("Frame")
frame:RegisterEvent("PLAYER_GUILD_UPDATE")
frame:RegisterEvent("PLAYER_LOGIN")


function GuildWho_OnEvent(self, event, ...)
	if IsInGuild() then
		GuildWho_Debug("event: "..event)
	else
		-- you aren't in a guild, return
		GuildWho_Debug("Not in a guild.")
		return
	end

	if event == "CHAT_MSG_SYSTEM" then
		local patterns = {
			"^"..ERR_FRIEND_ONLINE_SS:format("([^|]+)", ""):gsub("%.", "%%."):gsub("%[%]", ".+").."$",
			"^"..ERR_FRIEND_OFFLINE_S:format("(%S+)"):gsub("%.", "%%.").."$",
			"^"..ERR_GUILD_JOIN_S:format("(%S+)"):gsub("%.", "%%.").."$",
			"^"..ERR_GUILD_LEAVE_S:format("(%S+)"):gsub("%.", "%%.").."$",
			"^"..ERR_GUILD_REMOVE_SS:format("(%S+)", "(%S+)"):gsub("%.", "%%.").."$",
		}
		
		-- match the CHAT_MSG_SYSTEM message against global strings
		for i = 1, #patterns do
			local name = strmatch(select(1, ...), patterns[i])
			if name and guildRosterTable and (guildRosterTable[name] or i > 2) then
				if i >= 4 then
					-- message matches a character leaving or being kicked out of the guild, remove from guild roster table
					guildRosterTable[name] = nil
				elseif i == 2 then
					-- someone going offline
					guildRosterTable[name]["online"] = nil
				else
					if i == 3 then
						-- someone joins the guild, add to guild roster table
						guildRosterTable[name] = {}
					end
					guildRosterTable[name]["online"] = 1
				end
				if i <= 3 then
					GuildWho_Debug("guildRosterTable["..name.."] = {")
					GuildWho_Debug("    online = "..tostring(guildRosterTable[name]["online"]))
					GuildWho_Debug("    note = "..tostring(guildRosterTable[name]["note"]))
					GuildWho_Debug("}")
				else
					GuildWho_Debug("guildRosterTable["..name.."] = "..tostring(guildRosterTable[name]))
				end
				break
			end
		end
	elseif event == "GUILD_ROSTER_UPDATE" then
		if delayedCall then
			-- guild roster update was available for this call, so update local guild roster first
			GuildWho_UpdateGuildRoster()
			GuildWho_Debug("Delayed call")
			_G["GuildWho_SlashHandler"..delayedCall[1]](delayedCall[2])
			delayedCall = nil
		end
		if not rosterCooldown then
			-- calling GuildRoster() has a 10 sec "cooldown"
			frame:SetScript("OnUpdate", function(self, elapsed)
				if not self.lastUpdate then
					self.lastUpdate = 0
				end
				self.lastUpdate = self.lastUpdate + elapsed
				if self.lastUpdate > 10 then
					GuildWho_Debug("Roster cooldown finished")
					rosterCooldown = false
					self.lastUpdate = 0
					self:SetScript("OnUpdate", nil)
				end
			end)
			GuildWho_Debug("Roster cooldown start")
			rosterCooldown = true
		end
	else
		local guildName = (GetGuildInfo("player"))
		GuildWho_Debug(guildName)
		
		-- load/create saved variables if we are in a guild, and the guild name can be retrieved
		if guildName and IsInGuild() then
			if GuildWhoData then
				if GuildWhoData[GetRealmName()] then
					GuildWhoData[GetRealmName()][guildName] = GuildWhoData[GetRealmName()][guildName] or "officer"
				else
					GuildWhoData[GetRealmName()] = {
						[guildName] = "officer",
					}
				end
			else
				GuildWhoData = {
					[GetRealmName()] = {
						[guildName] = "officer",
					}
				}
			end
			frame:RegisterEvent("CHAT_MSG_SYSTEM")
			frame:RegisterEvent("GUILD_ROSTER_UPDATE")
			frame:UnregisterEvent("PLAYER_GUILD_UPDATE")
			GuildRoster()
		end
	end
end

frame:SetScript("OnEvent", GuildWho_OnEvent)


function GuildWho_SlashHandlerWho(msg)
	if not msg or msg == "" then
		GuildWho_PrintHelp()
	elseif strlower(msg) == "debug" then
		GuildWho_DebugMode()
	elseif strlower(msg) == "setnote" then
		local note = GuildWhoData[GetRealmName()]
		if note[GetGuildInfo("player")] == "officer" then
			note[GetGuildInfo("player")] = "public"
		else
			note[GetGuildInfo("player")] = "officer"
		end
		GuildWho_Print("Now using "..note[GetGuildInfo("player")].." note.")
		GuildWho_UpdateGuildRoster()
	else
		if rosterCooldown or delayedCall then
			-- roster on cooldown or a delayed call, get data now
			local lookup, altChar = GuildWho_GetLookup(msg, true)
			
			GuildWho_Debug(" ----- ")
			if rosterCooldown then
				GuildWho_Debug("Roster on cooldown, get data now")
			end
			
			if not lookup then
				-- character wasn't a guild match
				GuildWho_Print("No character named "..strupper(strsub(msg, 1, 1))..strlower(strsub(msg, 2)).." found in guild.")
				return
			end
			
			-- print result
			if altChar then
				local asAlt
				-- make a link outta the character name
				if lookup ~= altChar then
					asAlt = " as ".."|Hplayer:"..altChar.."|h["..altChar.."]|h"
				else
					lookup = "|Hplayer:"..lookup.."|h["..lookup.."]|h"
					asAlt = ""
				end
				GuildWho_Print(lookup.." is online"..asAlt..".")
			else
				GuildWho_Print(lookup.." is offline.")
			end
		else
			-- roster cooldown is ready - delaying call
			GuildWho_Debug("Roster cooldown is ready - delaying call")
			delayedCall = {"Who", msg}
			GuildRoster()
		end
	end
end

SlashCmdList["GWHO_WHO"] = GuildWho_SlashHandlerWho
SLASH_GWHO_WHO1 = "/gwho"


function GuildWho_SlashHandlerAlts(msg)
	if not msg or msg == "" then
		GuildWho_PrintHelp()
	else
		if rosterCooldown or delayedCall then
			-- roster on cooldown or a delayed call, get data now
			
			-- get main character of the given character
			local lookup, _, mainChar = GuildWho_GetLookup(msg)
			local hasAlts, header
			
			GuildWho_Debug(" ----- ")
			GuildWho_Debug("Roster on cooldown, get data now")
			
			if lookup then
				-- find alts of mainChar
				for character in pairs(guildRosterTable) do
					if (guildRosterTable[character]["note"] == mainChar or character == mainChar) and character ~= lookup then
						-- character is mainChar or an alt of mainChar, and not the searched character, so it's a match!
						hasAlts = true
						if not header then
							GuildWho_Print("Known alts of "..lookup..":")
							header = true
						end
						GuildWho_Print("  "..character)
					end
				end
				if not hasAlts then
					GuildWho_Print(lookup.." has no known alts.")
				end
			else
				-- searched character wasn't a guild match
				GuildWho_Print("No character named "..strupper(strsub(msg, 1, 1))..strlower(strsub(msg, 2)).." found in guild.")
			end
		else
			GuildWho_Debug("Roster cooldown is ready - delaying call")
			delayedCall = {"Alts", msg}
			GuildRoster()
		end
	end
end

SlashCmdList["GWHO_ALTS"] = GuildWho_SlashHandlerAlts
SLASH_GWHO_ALTS1 = "/alts"


hooksecurefunc("UnitPopup_OnClick", function(self)
	local dropdownFrame = _G[UIDROPDOWNMENU_INIT_MENU]
	local name = dropdownFrame.name
	
	if name and self.value == "GW_LISTALTS" then
		GuildWho_SlashHandlerAlts(name)
	end
end)


hooksecurefunc("UnitPopup_HideButtons", function()
	local dropdownMenu = _G[UIDROPDOWNMENU_INIT_MENU]
	local showEntry
	
	-- make sure the custom guild roster is initiated by now
	if not guildRosterTable and GetNumGuildMembers(true) > 0 then
		GuildWho_UpdateGuildRoster()
	end
	
	for index, value in ipairs(UnitPopupMenus[dropdownMenu.which]) do
		if value == "GW_LISTALTS" then
			-- only show this dropdown menu entry if the character is in the guild
			if guildRosterTable and guildRosterTable[dropdownMenu.name] then
				showEntry = 1
			else
				showEntry = 0
			end
			UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = showEntry
			break
		end
	end
end)

-- add a "List alts" entry to unit popup menus
UnitPopupButtons["GW_LISTALTS"] = {text = "List alts", dist = 0, func = UnitPopup_OnClick}
tinsert(UnitPopupMenus["FRIEND"], #UnitPopupMenus["FRIEND"] - 1, "GW_LISTALTS")


function GuildWho_GetLookup(name, onlineCheck)
	local lookup, altChar, mainChar
	
	GuildWho_Debug(" ----- ")
	
	for character in pairs(guildRosterTable) do
		-- does the provided search string match a guild member's name?
		if strlower(name) == strlower(character) then
			lookup = character
			GuildWho_Debug("lookup = "..lookup)
			
			-- search string matches an online character, so no need to look any further
			if guildRosterTable[character]["online"] and onlineCheck then
				return lookup, character
			end
			
			-- if this character's officer note is EPGPs or blank it's the main, else it's the name of the main
			mainChar = guildRosterTable[character]["note"] or character
			
			GuildWho_Debug("mainChar = "..mainChar)
			break
		end
	end
	
	if onlineCheck then
		for character in pairs(guildRosterTable) do
			if (guildRosterTable[character]["note"] == mainChar or character == mainChar) and guildRosterTable[character]["online"] then
				altChar = character
				GuildWho_Debug("altChar = "..character)
				break
			end
		end
	end
	
	return lookup, altChar, mainChar
end


function GuildWho_UpdateGuildRoster()
	local numGuildies = GetNumGuildMembers(true)
	
	-- if custom guild roster is not yet initiated, and default guild roster has been loaded
	if not guildRosterTable and	numGuildies > 0 then
		GuildWho_Debug(numGuildies.. " guild members.")
		-- initiate custom guild roster, get data from original guild roster
		guildRosterTable = {}
	end
	
	GuildWho_Debug("Updating guildRosterTable.")
	
	for i = 1, numGuildies do
		local name, _, _, _, _, _, publicnote, officernote, online = GetGuildRosterInfo(i)
		if not name then break end
		local note
		if GuildWhoData[GetRealmName()][GetGuildInfo("player")] == "officer" then
			note = strmatch(officernote, "^(%a+)$")
		else
			note = strmatch(publicnote, "^(%a+)$")
		end
		guildRosterTable[name] = {
			online = online,
			note = note,
		}
	end
end


function GuildWho_Print(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg, 1, 1, 0)
end


-- debug message function, only visible in debug mode
function GuildWho_Debug(msg)
	if debugMode and msg then
		DEFAULT_CHAT_FRAME:AddMessage("• "..msg, 1, 1, 0)
	end
end


-- function to toggle debug mode
function GuildWho_DebugMode()
	if not debugMode then
		debugMode = true
		GuildWho_Print("GuildWho: Debug mode ON.")
	else
		debugMode = false
		GuildWho_Print("GuildWho: Debug mode OFF.")
	end
end


function GuildWho_PrintHelp()
	GuildWho_Print("- How to use GuildWho:")
	GuildWho_Print("/gwho character - Find who this character is online as.")
	GuildWho_Print("/alts character - List this character's alts.")
	GuildWho_Print("/gwho setnote - Toggle using public/officer note.")
	GuildWho_Print("/gwho debug - Toggle debug messages.")
end